@@ -1,3 +1,3 @@
 ### Deal with global variables note
 utils::globalVariables(c("set", "patid", "obsdate", "eventdate", "admidate", "dod", "indexdt", "value", "total", "count", "med", "drug", "data2", "data3", "data5", "data6",
-                         "dob", "smoking", "var_time"))
+                         "dob", "smoking", "var_time", "yob", "medcodeid"))
